library navarggen_generator;

import 'dart:async';
import 'package:analyzer/dart/element/element.dart';
import 'package:build/src/builder/build_step.dart';
import 'package:navarggen/navarggen.dart';
import 'package:source_gen/source_gen.dart';

class NavigationArgumentsGenerator
    extends GeneratorForAnnotation<NavigationArguments> {
  @override
  FutureOr<String> generateForAnnotatedElement(
      Element element, ConstantReader annotation, BuildStep buildStep) {
    if (element is! ClassElement) {
      final name = element.name;
      throw InvalidGenerationSourceError('Generator cannot target `$name`.',
          todo: 'Remove the Entity annotation from `$name`.', element: element);
    }
    final classElement = element as ClassElement;

    final className = element.displayName;

    StringBuffer code = StringBuffer();
    code.writeln("");
    code.writeln("class ${className}Args {");
    code.writeln("");
    final a = classElement.metadata.firstWhere((it) {
      return it.computeConstantValue().type.toString() == 'NavigationArguments';
    }).computeConstantValue();
    final arguments = a.getField('arguments').toMapValue();
    arguments.forEach((key, value) {
      var type;
      if (value.type.toString() == 'Type') {
        type = value.toTypeValue();
      } else if (value.type.toString() == 'String') {
        type = value.toStringValue();
      } else {
        throw InvalidGenerationSourceError(
            'Type of argument must be one of Type or String.');
      }
      code.writeln('\tfinal $type ${key.toStringValue()};');
    });
    code.writeln('');
    code.writeln(
        '${className}Args({${arguments.keys.map((it) => 'this.${it.toStringValue()}').join(', ')}});');
    code.writeln('');
    code.writeln('}');
    code.writeln('');

    return code.toString();
  }
}
